﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2010.10.17.
 * Idő: 13:58
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace TetheredSun.IO
{
	/// <summary>
	/// Description of IniDocument.
	/// </summary>
	public class IniDocument : List<IniSection>
	{
		public IniDocument(int capacity) : base(capacity) { }
		
		public IniDocument(IList<IniSection> list) : base(list) { }
		
		public IniDocument() : base(1) { }
		
		
		public IniSection this[string name] {
			get {
				for (int i = 0; i < this.Count; i++) {
					if (this[i].Name == name) return this[i];
				}
				
				return null;
			}
		}
		
		
		public static IniDocument Open(string path)
		{
			using (StreamReader reader = new StreamReader(path, Encoding.UTF8)) {
				string text = reader.ReadToEnd();
				IniDocument document;
				
				if (IniDocument.TryParse(text, out document)) {
					return document;
				} else {
					return null;
				}
			}
		}
		
		public void Save(string path)
		{
			using (StreamWriter writer = new StreamWriter(path, false, Encoding.UTF8)) {
				string text = this.ToString();
				
				if (!String.IsNullOrEmpty(text)) writer.Write(text);
			}
		}
		
		public override string ToString()
		{
			string text = String.Empty;
			
			foreach (IniSection section in this) {
				text += section.ToString();
			}
			
			return text;
		}
		
		public static bool TryParse(string text, out IniDocument document)
		{
			if (String.IsNullOrEmpty(text)) {
				document = null;
				return false;
			}
			
			int endIndex = 0;
			string remainder = text;
			IniSection section;
			
			document = new IniDocument();
			
			while (!String.IsNullOrEmpty(remainder)) {
				if (IniSection.TryParse(remainder, out section, out endIndex)) {
					document.Add(section);
				}
				
				if (endIndex < 0) break;
				remainder = (endIndex < remainder.Length - 1) ? remainder.Substring(endIndex + 1) : null;
			}
			
			return true;
		}
		
	}
}
